# encoding: cp1251

#######################################################################
# Purpose: get latest FlexiCapture build except OCRT and FlexiCapture
#          (in our team we get them from https://tfs/HQ/_git)
# Actions:
# delete all local folders except OCRT and FlexiCapture
# find remote_folder:
#   cd //lingvo/abbyy/FormReader/Build/Results/FC12
#   get to latest FlexiCapture 12.*
# copy from remote_folder all files and all folders except OCRT and FlexiCapture
#
# Author: Pavel Yudaev, 2019
# ABBYY 
#######################################################################


import sys
import os
import glob
import shutil
import datetime
from dirsync import sync
import subprocess


def main():

    #KEEP = ("OCRT", "FlexiCapture", sys.argv[0])
    REMOTE = r"//lingvo/abbyy/FormReader/Build/Results/FC12/FlexiCapture 12.*"
    REMOVE_LOCAL_BEFORE_UPDATE = False

    latest_fc_folder = glob.glob(REMOTE)[-1] 
    remote_contents = glob.glob(latest_fc_folder + "//*")
    local_contents = glob.glob(r"*")

    if not(len(sys.argv) == 2 and sys.argv[1] == "--force"):
        print(f"")
        print(f"Update local contents with latest build from {latest_fc_folder}?")
        print(f"All local contents ", end="")
        print(f"will be updated!")
        print(f"y/N:")
        ans = input()
        ans="y"
        print(ans)
        print("")
        if ans not in ("y", "Y"):
            sys.exit(0)
    
    if REMOVE_LOCAL_BEFORE_UPDATE:
        print("Removing local contents:")
        for f in local_contents:
            if f not in KEEP:
                if os.path.isfile(f):
                    print(f"os.remove({f})")
                    os.remove(f)
            else:
                print(f"shutil.rmtree({f})")
                shutil.rmtree(f)

    print(f"Updating from {latest_fc_folder}:\n")
    t_start = datetime.datetime.now()
    subprocess.run([os.path.join(latest_fc_folder, "GetBuild.cmd")], input=".", encoding='ascii')    
    t_end = datetime.datetime.now()

    print("Completed.")


if __name__ == "__main__":
    main() 
